package com.mortals.xhx.busiz;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.mortals.xhx.base.framework.config.InterceptorConfig;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.ApiRespCodeEnum;
import com.mortals.xhx.common.code.ApproveStateEnum;
import com.mortals.xhx.common.code.OneThingTypeEnum;
import com.mortals.xhx.common.keys.BusizParams;
import com.mortals.xhx.feign.flowable.IApiFlowInstanceFeign;
import com.mortals.xhx.feign.rsp.ApiResp;
import com.mortals.xhx.feign.runtime.StartProcessInstanceReq;
import com.mortals.xhx.module.member.model.MemberEntity;
import com.mortals.xhx.module.member.service.MemberService;
import com.mortals.xhx.module.onething.model.OnethingEntity;
import com.mortals.xhx.module.onething.service.OnethingService;
import lombok.extern.apachecommons.CommonsLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.Map;

/**
 * 一件事微信对外服务接口
 *
 * @author: zxfei
 * @date: 2021/8/24 20:28
 */
@RestController
@CommonsLog
@RequestMapping("/api/one/")
public class WxOneThingApiController {

    @Autowired
    private OnethingService onethingService;
    @Autowired
    private ParamService paramService;
    @Autowired
    private IApiFlowInstanceFeign apiFlowInstanceFeign;
    @Autowired
    private MemberService memberService;
    @Autowired
    private InterceptorConfig config;

    /**
     * 微信用户提交材料
     *
     * @param entity
     * @return
     */
    @PostMapping("thing/create")
    public ApiResp<String> createThing(@RequestBody OnethingEntity entity) {
        ApiResp rsp = new ApiResp<>();
        rsp.setMsg(ApiRespCodeEnum.SUCCESS.getLabel());
        rsp.setCode(ApiRespCodeEnum.SUCCESS.getValue());
        try {
            entity.setType(OneThingTypeEnum.UNSUBMIT.getValue());
            entity.setStatus(ApproveStateEnum.UN_AUDIT.getValue());
            entity.setCreateUser("admin");
            entity.setCreateTime(new Date());
            OnethingEntity save = onethingService.save(entity);
            rsp.setData(save.getId());
        } catch (Exception e) {
            log.error("流程实例化终止失败", e);
            rsp.setCode(ApiRespCodeEnum.FAILED.getValue());
            rsp.setMsg(e.getMessage());
            return rsp;
        }

        log.info("响应【材料创建】【响应体】-->  " + JSONObject.toJSONString(rsp));
        return rsp;
    }

    /**
     * 发起流程
     *
     * @param req
     * @return
     */
    @PostMapping("thing/start")
    public ApiResp<String> startProcess(@RequestBody StartOneThingReq req) {
        ApiResp rsp = new ApiResp<>();
        rsp.setMsg(ApiRespCodeEnum.SUCCESS.getLabel());
        rsp.setCode(ApiRespCodeEnum.SUCCESS.getValue());
        try {
            OnethingEntity onethingEntity = onethingService.get(req.getOneThingId());
            MemberEntity memberEntity = memberService.get(onethingEntity.getMemberId());

            //获取流程定义key 发起流程 当前通过param参数固定获取
            String oneThingKey = paramService.getValueByKey(BusizParams.SYS_PARAM_FLOW_KEY);
            //通过流程key获取基础提交参数
            String vars = paramService.getValueByKey(oneThingKey);

            ObjectMapper mapper = new ObjectMapper();
            Map<String,Object> variables = mapper.readValue(vars, Map.class);

            StartProcessInstanceReq startProcessInstanceReq = new StartProcessInstanceReq();
            startProcessInstanceReq.setProcessDefinitionKey(oneThingKey);
            startProcessInstanceReq.setBusinessKey(onethingEntity.getId().toString());
            startProcessInstanceReq.setFormName("一件事一次办");
            startProcessInstanceReq.setUserCode(memberEntity.getOpenId());
            startProcessInstanceReq.setPlatformSn(config.getTenantId());
            startProcessInstanceReq.setVariables(variables);

            ApiResp<String> resp = apiFlowInstanceFeign.processStart(startProcessInstanceReq);
            onethingEntity.setProcessInstanceId(resp.getData());
            onethingEntity.setType(OneThingTypeEnum.SUBMIT.getValue());
            onethingEntity.setStatus(ApproveStateEnum.AUDITING.getValue());

            onethingService.update(onethingEntity);

            rsp.setData(resp.getData());

        } catch (Exception e) {
            log.error("发起审批流程", e);
            rsp.setCode(ApiRespCodeEnum.FAILED.getValue());
            rsp.setMsg(e.getMessage());
            return rsp;
        }
        log.info("响应【材料创建】【响应体】-->  " + JSONObject.toJSONString(rsp));
        return rsp;
    }


}
